#!/usr/bin/env python

"""
See README in this directory for the semantics of the example.  This file
constructs the credentials described and puts copies into this directory

cmd1:env keystore=`pwd` ./attr.py 
"""

import os
import ABAC

ctxt = ABAC.Context()
print "ABAC version %s" % ctxt.version()

# Keystore is the directory containing the principal credentials.
# Load existing principals and/or policy credentials
if (os.environ.has_key("keystore")) :
    keystore=os.environ["keystore"]
    ctxt.load_directory(keystore)

out = ctxt.context_principals()
print "...initial principal set..."
for x in out[1]:
    print "%s " % x.string()
print "\n" 

out = ctxt.context_credentials()
print "...initial policy attribute set..."
for c in out[1]:
    print "%s <- %s" % (c.head_string(), c.tail_string())
print "\n"

# retrieve principals' keyid value from local credential files
geniID=ABAC.ID("Geni_ID.pem");
geniID.id_load_privkey_file("Geni_private.pem");
geni=geniID.id_keyid()

bobID=ABAC.ID("Bob_ID.pem");
bobID.id_load_privkey_file("Bob_private.pem");
bob=bobID.id_keyid()

jackID=ABAC.ID("Jack_ID.pem");
jackID.id_load_privkey_file("Jack_private.pem");
jack=jackID.id_keyid()

joeID=ABAC.ID("Joe_ID.pem");
joeID.id_load_privkey_file("Joe_private.pem");
joe=joeID.id_keyid()

################################################
# Credential 1, 
# [keyid:geni].role:leader
#         <- [keyid:geni].role:equivalent([principal:?P[keyid:geni].role:leader])
head=ABAC.Role(geni,"leader")

# initialize the role constraint on a principlal
condrole=ABAC.Role(geni,"leader")
cond=ABAC.Constraint(condrole)

# make the data term with the role constraint
param=ABAC.DataTerm("principal","P", cond)
tail = ABAC.Role(geni,"equivalent")
tail.role_add_data_term(param)

# build he attribute policy
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)

# finalize the policy
attr.attribute_bake()

# write the policy out to a credential file
attr.attribute_write_cert("geni_leader__geni_leader_qP_attr.der")

# load the policy into the context using the credential file
ctxt.load_attribute_file("geni_leader__geni_leader_qP_attr.der")
print attr.string() 
print attr.typed_string()
print "\n"

#################################################
# Credential 2
# [keyid:geni].role:leader <- [keyid:bob]
head=ABAC.Role(geni,"leader")
tail = ABAC.Role(bob)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("geni_leader__Bob_attr.der")
ctxt.load_attribute_file("geni_leader__Bob_attr.der")
print attr.string() 
print attr.typed_string()
print "\n"

#################################################
# Credential 3
# [keyid:geni].role:equivalent([keyid:bob]) <- [keyid:Joe]
param=ABAC.DataTerm(bobID)
head = ABAC.Role(geni,"equivalent")
head.role_add_data_term(param)
tail = ABAC.Role(joe)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("geni_equivalent_Bob__Joe_attr.der")
ctxt.load_attribute_file("geni_equivalent_Bob__Joe_attr.der")
print attr.string() 
print attr.typed_string()
print "\n"

#################################################
# Credential 4
# [keyid:geni].role:equivalent([keyid:Joe]) <- [keyid:Bob]
param=ABAC.DataTerm(joeID)
head = ABAC.Role(geni,"equivalent")
head.role_add_data_term(param)
tail = ABAC.Role(bob)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("geni_equivalent_Joe__Bob_attr.der")
ctxt.load_attribute_file("geni_equivalent_Joe__Bob_attr.der")
print attr.string() 
print attr.typed_string()
print "\n"
